use strict;
use warnings;

BEGIN { $| = 1;
	use Test::More;
	plan tests => 12;
}


use Bio::EnsEMBL::Registry;
use Bio::EnsEMBL::Test::TestUtils;
use Bio::EnsEMBL::Variation::IndividualGenotype;
use Bio::EnsEMBL::Variation::Variation;
use Bio::EnsEMBL::Variation::Individual;



# test constructor
my $ind = Bio::EnsEMBL::Variation::Individual->new
  (-name => 'test individual',
   -description => 'This is a test individual',
   -gender => 'Male');

my $var = Bio::EnsEMBL::Variation::Variation->new
  (-name => 'rs123',
   -synonyms => {'dbSNP' => ['ss12', 'ss144']},
   -source => 'dbSNP');

my $genotype = ['A','C'];
my $subsnp   ='ss12';


my $ind_gtype = Bio::EnsEMBL::Variation::IndividualGenotype->new
  (-genotype => $genotype,
   -variation => $var,
   -individual => $ind,
   -subsnp     => $subsnp  
);


ok($ind_gtype->individual()->name() eq $ind->name(), "ind name");
ok($ind_gtype->variation()->name()  eq $var->name(), "var name");

ok($ind_gtype->allele1() eq 'A', "allele 1");
ok($ind_gtype->allele2() eq 'C', "allele 2");

ok($ind_gtype->genotype() eq $genotype, "genotype");
my $string = join '|', @{$genotype};
ok($ind_gtype->genotype_string() eq $string, "genotype string");

ok($ind_gtype->subsnp() eq $subsnp, "subsnp");

ok($ind_gtype->ambiguity_code() eq 'M', "ambiguity code"); 

# test getter/setters

ok(test_getter_setter($ind_gtype, 'allele1', 'TT'), "get/set allele 1");
ok(test_getter_setter($ind_gtype, 'allele2', '-'),  "get/set allele 2");

my $ind2 = Bio::EnsEMBL::Variation::Individual->new
  (-name => 'test individual 2',
   -description => 'This is a second test individual',
   -gender => 'Female');

my $var2 = Bio::EnsEMBL::Variation::Variation->new
  (-name => 'rs332',
   -synonyms => {'dbSNP' => ['ss44', 'ss890']},
   -source => 'dbSNP');

ok(test_getter_setter($ind_gtype, 'individual', $ind2), "get/set individual");
ok(test_getter_setter($ind_gtype, 'variation', $var2),  "get/set variation");


done_testing();
